'''
Synthetic instance generator for Materials Discovery
Copyright 2014, Institute for Computational Sustainability
See README for license and citation information.  

Main contributors: 
Ronan Le Bras (Cornell University Dept. of Computer Science) lebras@cs.cornell.edu
Richard Bernstein (Cornell University Dept. of Computer Science) rab38@cornell.edu
John Gregoire (Caltech JCAP Energy Innovation Hub) gregoire@caltech.edu

SyntheticGenerator.py - outer script for generating synthetic instances
Modified 11/06/2014
'''
import random
import argparse
from SyntheticXRD import *
from SyntheticModels import *

#Handle command-line arguments
parser = argparse.ArgumentParser(description='Synthetic instance generator for Materials Discovery')
parser.add_argument('-o',action='store',dest='path',help='Output path for instance files',required=True)
parser.add_argument('-b',action='store',dest='basename',help='Base filename for instance files',required=True)
parser.add_argument('-n',action='store',dest='n',help='Set the number of composition points for each element (n>=2)',type=int,required=True)
parser.add_argument('-r',action='store',dest='r',help='Set the number of peaks to remove randomly (default=0)',type=int,required=False)
parser.add_argument('-t',action='store',dest='type',help='Model type: allife (default) or abc',required=False)
parser.add_argument('-p',action='store',dest='p',help='Model peak proportion: integer percentage of total model peaks to include (default 100 means \'all\', 0 means \'1 per phase\')',type=int,required=False);
parser.add_argument('-s',action='store',dest='s',help='PRNG seed to use for randomized components (0 means use default source)',type=int,required=False)
parser.add_argument('-i',action='store',dest='intensity',help='Height intensity scale (default=100)',type=float,required=False) 
arguments = parser.parse_args()

if arguments.s:
    random.seed(arguments.s)

#Output location and filename
folder = arguments.path
name=arguments.basename+'_n%d' %(arguments.n)
name += '_p%d' %(arguments.p)
if arguments.r:
    name += '_r%d'%(arguments.r)
else:
    arguments.r=0
    
if not arguments.type:
    arguments.type='allife'
modelDescription = "Synthetic type:{0:s}, p:{1:d}, n:{2:d}, r:{3:d}".format(arguments.type,arguments.p,arguments.n,arguments.r,arguments.intensity)
if arguments.s:
    name += '_s%d'%(arguments.s)
    modelDescription += ", s:{0:d}".format(arguments.s)
if arguments.intensity:
    modelDescription += ', i:{0:1.3f}'.format(arguments.intensity)
else:
    arguments.intensity=100.

#qgrid defines the diffraction pattern x axis. [lowest Q value, interval, number of values]
qgrid = [15.0, .1, 650]

#build the model
model = getModelByArgument(arguments)
synthpeaks, synthpatterns, synthphases, synthmergedpeaks=calcxrd(model.phases, model.regions, model.export_comp, model.export_region, qgrid)
if arguments.r>0:
    synthpeaks, synthpatterns, removedpointpk = removepeaks(synthpeaks, arguments.r, qgrid)

#output the instance and solution info
saveInstance(folder, name, model.elements, synthpatterns, synthpeaks, synthphases, model.export_comp, qgrid,modelDescription,model.uuid,arguments.intensity)
saveSolution(folder, name, synthphases, [model.regions[model.export_region[i]] for i in xrange(model.nsamples)], model.phases, model.export_comp,qgrid, model.qbasisoffsets,modelDescription,model.uuid,arguments.intensity)

