'''
Synthetic instance generator for Materials Discovery
Copyright 2014, Institute for Computational Sustainability
See README for license and citation information.  

Main contributors: 
Ronan Le Bras (Cornell University Dept. of Computer Science) lebras@cs.cornell.edu
Richard Bernstein (Cornell University Dept. of Computer Science) rab38@cornell.edu
John Gregoire (Caltech JCAP Energy Innovation Hub) gregoire@caltech.edu

SyntheticModels.py - holds model parameters for two kinds of synthetic material spreads
Modified 11/06/2014
'''

from SyntheticXRD import *
import uuid
import random

class synthModel(object):
    """
    Class standardizing the basic info needed for a model
    """
    nsamples=None
    qbasisoffsets=None
    phases=None
    regions=None
    export_region=None
    export_comp=None
    elements=None
    uuid=None
    def __init__(self,compsize,regionchoice,qbasisoffsets,phases,regions,elements):
        self.qbasisoffsets=qbasisoffsets
        self.phases=phases
        self.export_comp = compgrid(compsize, ternary=True)
        self.regions = regions
        self.export_region = [regionchoice(c) for c in self.export_comp]
        self.nsamples=len(self.export_comp)
        self.elements=elements
        self.uuid = str(uuid.uuid4())

def getModelByArgument(arguments):
    """
    Just calls the correct model-generating function for each model type
    """
    if not arguments.type or arguments.type=='allife':
        return getModelAlLiFe(arguments)
    elif arguments.type=='abc':
        return getModelABC(arguments)
    else:
        raise Exception("Invalid model type: "  + arguments.type + ", expected allife or abc")
      
def selectPeakSubset(peaksByPhase, intPercentageKept):
    """
    Select a subset of peaks from the system, including at least one peak from each phase, 
    and using the provided percentage to set the number of additional peaks selected with
    uniform chance across all phases. Allows for the randomized creation of simpler instances
    based on a given prototype.
    """
    if not (intPercentageKept>=0 and intPercentageKept <=100):raise ValueError("percentage must be between 0 and 100")
    numPhases = len(peaksByPhase)
    numPeaksByPhase = [len(peaksByPhase[i]) for i in xrange(numPhases)]
    peakIndexPairs=[(i,j) for i in xrange(numPhases) for j in xrange(numPeaksByPhase[i])]
    selectedPeaksByPhase = [[False for j in xrange(numPeaksByPhase[i])] for i in xrange(numPhases)]
    for i in xrange(numPhases):
        if numPeaksByPhase[i]:
            sel = random.randrange(numPeaksByPhase[i])
            selectedPeaksByPhase[i][sel]=True
            peakIndexPairs.remove((i,sel))
    numToKeep = int(round(len(peakIndexPairs)*intPercentageKept/100.))
    selList = sorted(random.sample(range(len(peakIndexPairs)),numToKeep))
    for i in selList: selectedPeaksByPhase[peakIndexPairs[i][0]][peakIndexPairs[i][1]]=True
    finalSelections = [[peaksByPhase[i][j] for j in xrange(numPeaksByPhase[i]) if selectedPeaksByPhase[i][j]] for i in xrange(numPhases)]
    return finalSelections
    
def getModelAlLiFe(arguments):
    """
    Generate model information for a synthetic composition spread based on a realistic
    theoretical model of a mixture of Al, Li and Fe.
    """
    elements = ["Al","Li","Fe"]
    compsize = arguments.n if arguments.n else 2

    #define boundaries
    c01=lambda x: 0.18+x[1]*0.37/0.17
    c12=lambda x: 0.9-x[1]*0.35/0.17
    a12=lambda x: 0.1+x[1]*0.18/0.17
    c34=lambda x: 0.65-(x[1]-0.17)*0.65/0.33
    c45=lambda x: 0.65-(x[1]-0.17)*0.15/0.33
    a45=lambda x: 0.18-(x[1]-0.17)*0.18/0.33
    
    def regionchoice(x):
        """
        Returns the phase map region for a given composition mixture
        """
        if x[1]<.17:
            if x[2]<c01(x):
                return 0
            elif x[2]<c12(x):
                return 1
            else:
                return 2
        elif x[1]<0.5:
            if x[2]<c34(x):
                return 3
            elif x[2]<c45(x):
                return 4
            else:
                return 5
        else:
            return 6

    #start positions of peaks
    qalpha_1_0_0=18.05
    qbeta_0_1_0=23.60
    qgamma_0_0_1=17.06
    qdelta_83_17_0=11.24
    qepsilon_5_5_0=18.7
    qzeta_0_5_5=12.98

    #list of start position of the first peak in each phase    
    qbasisoffsets = [qalpha_1_0_0,qbeta_0_1_0,qgamma_0_0_1,qdelta_83_17_0,qepsilon_5_5_0,qzeta_0_5_5]
    
    #lattice constant shifts
    qalpha_perc=(qgamma_0_0_1-qalpha_1_0_0)/qalpha_1_0_0
    qdelta_perc=(10.67-qdelta_83_17_0)/qdelta_83_17_0/0.66 # the .66 is this full ranfe of Fe composition in the phase and 10.67 is the 1st peak position for Fe5LiO8
    qgamma_pera=-1.*qalpha_perc
    
    #Peak locations for each phase, relative to the first peak in each
    Al2O3=[[1.000,1.000,1.000],[1.365,1.508,1.000],[1.463,0.697,1.000],[1.607,0.008,1.000],[1.669,1.440,1.000],[1.772,0.023,1.000],[2.001,0.706,1.000],[2.173,1.370,1.000],[2.250,0.035,1.000],[2.298,0.050,1.000],[2.304,0.128,1.000],[2.478,0.526,1.000],[2.533,0.796,1.000],[2.606,0.017,1.000],[2.728,0.021,1.000],[2.810,0.222,1.000],[2.821,0.127,1.000],[2.918,0.011,1.000],[2.925,0.087,1.000],[3.001,0.009,1.000],[3.035,0.068,1.000],[3.056,0.005,1.000],[3.091,0.051,1.000],[3.097,0.044,1.000],[3.167,0.095,1.000],[3.214,0.024,1.000],[3.229,0.115,1.000],[3.328,0.006,1.000],[3.337,0.217,1.000],[3.419,0.027,1.000],[3.488,0.158,1.000],[3.530,0.005,1.000],[3.545,0.029,1.000],[3.578,0.002,1.000],[3.691,0.006,1.000],[3.699,0.005,1.000],[3.723,0.045,1.000],[3.792,0.030,1.000],[3.834,0.145,1.000],[3.846,0.109,1.000],[3.872,0.083,1.000],[3.916,0.005,1.000],[3.951,0.047,1.000],[4.001,0.030,1.000],[4.057,0.161,1.000],[4.095,0.059,1.000],[4.115,0.065,1.000],[4.129,0.006,1.000],[4.139,0.003,1.000],[4.189,0.238,1.000],[4.240,0.002,1.000],[4.261,0.008,1.000],[4.276,0.030,1.000],[4.314,0.094,1.000],[4.346,0.003,1.000],[4.357,0.065,1.000]]
    
    LiO2=[[1.000,1.000,1.000],[1.155,0.058,1.000],[1.633,0.533,1.000],[1.915,0.150,1.000],[2.000,0.001,1.000],[2.309,0.047,1.000],[2.516,0.035,1.000],[2.582,0.001,1.000],[2.829,0.074,1.000],[3.001,0.024,1.000],[3.266,0.025,1.000]]
    
    Fe2O3=[[1.000,1.000,1.000],[1.364,3.333,1.000],[1.462,2.333,1.000],[1.607,0.100,1.000],[1.669,0.667,1.000],[1.773,0.100,1.000],[2.001,1.333,1.000],[2.175,1.500,1.000],[2.250,0.033,1.000],[2.298,0.167,1.000],[2.304,0.333,1.000],[2.479,1.000,1.000],[2.534,1.000,1.000],[2.605,0.033,1.000],[2.729,0.100,1.000],[2.810,0.333,1.000],[2.821,0.200,1.000],[2.926,0.267,1.000],[3.000,0.133,1.000],[3.035,0.067,1.000],[3.096,0.167,1.000],[3.168,0.167,1.000],[3.229,0.233,1.000],[3.337,0.233,1.000],[3.421,0.067,1.000],[3.489,0.233,1.000],[3.532,0.033,1.000],[3.546,0.100,1.000],[3.725,0.133,1.000],[3.794,0.033,1.000],[3.834,0.167,1.000],[3.846,0.133,1.000],[3.870,0.167,1.000],[3.953,0.067,1.000],[4.000,0.067,1.000],[4.057,0.167,1.000],[4.093,0.033,1.000],[4.116,0.100,1.000],[4.191,0.200,1.000],[4.259,0.033,1.000],[4.314,0.100,1.000],[4.365,0.167,1.000],[4.391,0.100,1.000],[4.554,0.133,1.000],[4.599,0.133,1.000]]
    
    LiAl5O8=[[1.000,1.000,1.000],[1.225,0.339,1.000],[1.581,0.936,1.000],[1.732,0.655,1.000],[2.000,2.310,1.000],[2.121,0.035,1.000],[2.236,0.111,1.000],[2.345,5.848,1.000],[2.450,0.415,1.000],[2.549,0.041,1.000],[2.646,0.035,1.000],[2.828,2.673,1.000],[2.915,0.018,1.000],[2.999,0.041,1.000],[3.082,0.012,1.000],[3.239,0.181,1.000],[3.316,0.006,1.000],[3.464,0.643,1.000],[3.534,0.018,1.000],[3.605,0.047,1.000],[3.674,1.760,1.000],[3.807,0.117,1.000],[3.872,0.175,1.000],[4.000,3.322,1.000],[4.061,0.018,1.000],[4.124,0.029,1.000],[4.183,0.018,1.000],[4.243,0.006,1.000],[4.302,0.035,1.000],[4.359,0.058,1.000],[4.470,0.211,1.000],[4.528,0.018,1.000],[4.584,0.023,1.000],[4.637,0.392,1.000],[4.692,0.006,1.000],[4.744,0.058,1.000],[4.797,0.006,1.000],[4.897,0.216,1.000],[4.950,0.018,1.000],[4.998,0.018,1.000],[5.048,0.018,1.000]]#there are a bunch more peaks in Fe5LiO8
    
    LiAlO2=[[1.000,1.000,1.000],[1.982,0.151,1.000],[2.000,0.004,1.000],[2.065,0.001,1.000],[2.366,0.810,1.000],[2.569,0.019,1.000],[2.999,0.001,1.000],[3.044,0.106,1.000],[3.307,0.160,1.000],[3.385,0.169,1.000],[3.529,0.067,1.000],[3.865,0.007,1.000],[3.923,0.011,1.000],[3.933,0.012,1.000],[3.966,0.002,1.000],[3.999,0.017,1.000],[4.128,0.062,1.000],[4.153,0.033,1.000],[4.250,0.003,1.000]]
    
    LiFeO2=[[1.000,1.000,1.000],[1.921,0.345,1.000],[2.000,0.045,1.000],[2.006,0.115,1.000],[2.315,0.619,1.000],[2.521,0.105,1.000],[3.004,0.102,1.000],[3.270,0.145,1.000],[3.277,0.217,1.000],[3.425,0.091,1.000],[3.799,0.024,1.000],[3.832,0.015,1.000],[3.838,0.029,1.000],[4.000,0.018,1.000],[4.013,0.058,1.000],[4.126,0.028,1.000],[4.137,0.026,1.000],[4.444,0.023,1.000],[4.627,0.028,1.000],[4.727,0.005,1.000],[5.000,0.005,1.000],[5.016,0.016,1.000],[5.036,0.006,1.000],[5.042,0.006,1.000],[5.052,0.009,1.000],[5.171,0.037,1.000],[5.182,0.045,1.000],[5.267,0.012,1.000],[5.278,0.013,1.000],[5.525,0.018,1.000],[5.661,0.015,1.000],[5.674,0.040,1.000],[5.674,0.022,1.000],[5.755,0.004,1.000],[5.762,0.015,1.000],[5.975,0.022,1.000],[5.998,0.002,1.000],[6.005,0.004,1.000],[6.012,0.008,1.000],[6.020,0.007,1.000]]
    
    #           alpha    beta  gamma  delta    epsilon   zeta
    phases=[Al2O3, LiO2, Fe2O3, LiAl5O8, LiAlO2, LiFeO2]
    if arguments.intensity:
        phases = [[[pk[0],pk[1]*arguments.intensity,pk[2]] for pk in ph] for ph in phases]
    phases=selectPeakSubset(phases,arguments.p)

    #Functions that define the changes in each parameter, in each phase region, as a function of composition    
    r0_fc_alpha=lambda x: x[2]*0.18/c01(x)
    r0_fc_delta=lambda x: x[2]*0.55/c01(x)
    r0_qalpha=lambda x: qalpha_1_0_0*(1.+qalpha_perc*r0_fc_alpha(x))
    r0_qdelta=lambda x: qdelta_83_17_0*(1.+qdelta_perc*r0_fc_delta(x))
    r0_fdelta=lambda x: x[1]/0.17
    r0_falpha=lambda x: 1.-r0_fdelta(x)
    r0_walpha=lambda x: 1.5/(2.*r0_falpha(x)+1.0)
    r0_halpha=lambda x: r0_falpha(x)/r0_walpha(x)
    r0_wdelta=lambda x: 1.5/(2.*r0_fdelta(x)+1.0)
    r0_hdelta=lambda x: r0_fdelta(x)/r0_wdelta(x)
    
    r1_qalpha=lambda x: qalpha_1_0_0*(1.+qalpha_perc*.18)
    r1_qdelta=lambda x: qdelta_83_17_0*(1.+qdelta_perc*.55)
    r1_qgamma=lambda x: qgamma_0_0_1*(1.+qgamma_pera*.1)
    r1_fdelta=lambda x: x[1]/0.17
    r1_falpha=lambda x: (x[0]-0.1-0.18*r1_fdelta(x))/0.72
    r1_fgamma=lambda x: 1.-r1_fdelta(x)-r1_falpha(x)
    r1_walpha=lambda x: 1.5/(2.*r1_falpha(x)+1.0)
    r1_halpha=lambda x: r1_falpha(x)/r1_walpha(x)
    r1_wdelta=lambda x: 1.5/(2.*r1_fdelta(x)+1.0)
    r1_hdelta=lambda x: r1_fdelta(x)/r1_wdelta(x)
    r1_wgamma=lambda x: 1.5/(2.*r1_fgamma(x)+1.0)
    r1_hgamma=lambda x: r1_fgamma(x)/r1_wgamma(x)
    
    r2_fa_gamma=lambda x: x[0]*0.1/a12(x)
    r2_fc_delta=lambda x: .83-x[0]*0.28/a12(x)
    r2_qgamma=lambda x: qgamma_0_0_1*(1.+qgamma_pera*r2_fa_gamma(x))
    r2_qdelta=lambda x: qdelta_83_17_0*(1.+qdelta_perc*r2_fc_delta(x))
    r2_fdelta=lambda x: x[1]/0.17
    r2_fgamma=lambda x: 1.-r2_fdelta(x)
    r2_wdelta=lambda x: 1.5/(2.*r2_fdelta(x)+1.0)
    r2_hdelta=lambda x: r2_fdelta(x)/r2_wdelta(x)
    r2_wgamma=lambda x: 1.5/(2.*r2_fgamma(x)+1.0)
    r2_hgamma=lambda x: r2_fgamma(x)/r2_wgamma(x)
    
    r3_fc_delta=lambda x: x[2]*0.65/c34(x)
    r3_qepsilon=lambda x: qepsilon_5_5_0
    r3_qdelta=lambda x: qdelta_83_17_0*(1.+qdelta_perc*r3_fc_delta(x))
    r3_fepsilon=lambda x: (x[1]-0.17)/.33
    r3_fdelta=lambda x: 1.-r3_fepsilon(x)
    r3_wepsilon=lambda x: 1.5/(2.*r3_fepsilon(x)+1.0)
    r3_hepsilon=lambda x: r3_fepsilon(x)/r3_wepsilon(x)
    r3_wdelta=lambda x: 1.5/(2.*r3_fdelta(x)+1.0)
    r3_hdelta=lambda x: r3_fdelta(x)/r3_wdelta(x)
    
    r4_qdelta=lambda x: qdelta_83_17_0*(1.+qdelta_perc*.65)
    r4_qzeta=lambda x: qzeta_0_5_5
    r4_qepsilon=lambda x: qepsilon_5_5_0
    r4_fdelta=lambda x: 1.-(x[1]-0.17)/.33
    r4_fepsilon=lambda x: (x[0]-0.18*r4_fdelta(x))/0.5
    r4_fzeta=lambda x: 1.-r4_fdelta(x)-r4_fepsilon(x)
    r4_wepsilon=lambda x: 1.5/(2.*r4_fepsilon(x)+1.0)
    r4_hepsilon=lambda x: r4_fepsilon(x)/r4_wepsilon(x)
    r4_wdelta=lambda x: 1.5/(2.*r4_fdelta(x)+1.0)
    r4_hdelta=lambda x: r4_fdelta(x)/r4_wdelta(x)
    r4_wzeta=lambda x: 1.5/(2.*r4_fzeta(x)+1.0)
    r4_hzeta=lambda x: r4_fzeta(x)/r4_wzeta(x)
    
    
    r5_fc_delta=lambda x:  .83-x[0]*0.18/a45(x)
    r5_qzeta=lambda x: qzeta_0_5_5
    r5_qdelta=lambda x: qdelta_83_17_0*(1.+qdelta_perc*r5_fc_delta(x))
    r5_fzeta=lambda x: (x[1]-0.17)/.33
    r5_fdelta=lambda x: 1.-r5_fzeta(x)
    r5_wzeta=lambda x: 1.5/(2.*r5_fzeta(x)+1.0)
    r5_hzeta=lambda x: r5_fzeta(x)/r5_wzeta(x)
    r5_wdelta=lambda x: 1.5/(2.*r5_fdelta(x)+1.0)
    r5_hdelta=lambda x: r5_fdelta(x)/r5_wdelta(x)
    
    r6_qbeta=lambda x: qbeta_0_1_0
    r6_qzeta=lambda x: qzeta_0_5_5
    r6_qepsilon=lambda x: qepsilon_5_5_0
    r6_fbeta=lambda x: (x[1]-0.5)/.5
    r6_fepsilon=lambda x: x[0]/0.5
    r6_fzeta=lambda x: 1.-r6_fbeta(x)-r6_fepsilon(x)
    r6_wepsilon=lambda x: 1.5/(2.*r6_fepsilon(x)+1.0)
    r6_hepsilon=lambda x: r6_fepsilon(x)/r6_wepsilon(x)
    r6_wbeta=lambda x: 1.5/(2.*r6_fbeta(x)+1.0)
    r6_hbeta=lambda x: r6_fbeta(x)/r6_wbeta(x)
    r6_wzeta=lambda x: 1.5/(2.*r6_fzeta(x)+1.0)
    r6_hzeta=lambda x: r6_fzeta(x)/r6_wzeta(x)
    
    
    q0=lambda x: 1.
    h0=lambda x: 0.
    w0=lambda x: 1.
    
    #Set of model functions to use for each phase in each phase region    
    regions = [                                 \
    [[r0_qalpha, r0_halpha, r0_walpha], [q0, h0, w0], [q0, h0, w0], [r0_qdelta, r0_hdelta, r0_wdelta], [q0, h0, w0], [q0, h0, w0]]                      ,\
    [[r1_qalpha, r1_halpha, r1_walpha], [q0, h0, w0], [r1_qgamma, r1_hgamma, r1_wgamma], [r1_qdelta, r1_hdelta, r1_wdelta], [q0, h0, w0], [q0, h0, w0]]                      ,\
    [[q0, h0, w0], [q0, h0, w0], [r2_qgamma, r2_hgamma, r2_wgamma], [r2_qdelta, r2_hdelta, r2_wdelta], [q0, h0, w0], [q0, h0, w0]]                      ,\
    [[q0, h0, w0], [q0, h0, w0], [q0, h0, w0], [r3_qdelta, r3_hdelta, r3_wdelta], [r3_qepsilon, r3_hepsilon, r3_wepsilon], [q0, h0, w0]]                      ,\
    [[q0, h0, w0], [q0, h0, w0], [q0, h0, w0], [r4_qdelta, r4_hdelta, r4_wdelta], [r4_qepsilon, r4_hepsilon, r4_wepsilon], [r4_qzeta, r4_hzeta, r4_wzeta]]                      ,\
    [[q0, h0, w0], [q0, h0, w0], [q0, h0, w0], [r5_qdelta, r5_hdelta, r5_wdelta], [q0, h0, w0], [r5_qzeta, r5_hzeta, r5_wzeta]]                      ,\
    [[q0, h0, w0], [r6_qbeta, r6_hbeta, r6_wbeta], [q0, h0, w0], [q0, h0, w0], [r6_qepsilon, r6_hepsilon, r6_wepsilon], [r6_qzeta, r6_hzeta, r6_wzeta]]                      ,\
    ]

    #Object containing the results of the above manipulations
    return synthModel(compsize,regionchoice,qbasisoffsets,phases,regions,elements)

def getModelABC(arguments):
    """
    Generate model information for a synthetic composition spread of made-up materials,
    with arbitrary peak parameters, and having less variability than real systems
    (for example, no changes in peak width or peak shift with changes in composition).
    """
    elements=["A","B","C"]
    compsize = arguments.n if arguments.n else 2

    #define boundaries
    c01=lambda x: .25*x[1]/0.5
    a02=lambda x: .25*x[1]/0.5
    
    def regionchoice(x):
        """
        Returns the phase map region for a given composition mixture
        """
        if x[1]<.5 and x[0]>a02(x) and x[2]>c01(x):
            return 0
        elif x[0]>x[2]:
            return 1
        else:
            return 2
    
    #Peak locations for each phase, relative to the first peak in each
    alpha=[[1.0, 1.0, 1.0], [4./3., .5, 1.0], [5./3., .8, 1.0],[6./3, .2, 1.0],[7./3, .25, 1.0],[2.4, .2, 1.0],[2.5, .4, 1.0],[2.6, .6, 1.0],[2.8, .8, 1.0],[2.87, .6, 1.0]]
    beta=[[1.0, 1.0, 1.0], [4.5/3., 1., 1.0], [5./3., 1., 1.0], [6.5/3., 1., 1.0], [7./3.,0.8,1.0], [7.3/3.,0.8,1.0], [7.6/3.,0.65,1.0], [8.5/3.,0.75,1.0], [8.7/3.,0.5,1.0], [8.9/3.,0.1,1.0]]
    gamma=[[1.0, 1.0, 1.0], [3.5/3., .1, 1.0], [4./3., .5, 1.0], [5./3., .1, 1.0], [6./3., .8, 1.0], [6.2/3.,0.8,1.0],[6.6/3.,0.65,1.0], [6.85/3.,.25,1.0], [7./3., 1., 1.0], [7.5/3.,0.8,1.0]]
    delta=[[1.0, 1.0, 1.0], [1.07, .3, 1.0], [1.19, .4, 1.0], [1.3, .3, 1.0], [1.4, 1.5, 1.0], [1.8, .6, 1.0], [1.95, .8, 1.0],[2.05, .2, 1.0],[2.17, .25, 1.0],[2.3, .2, 1.0]]
    phases = [alpha, beta, gamma, delta]
    if arguments.intensity:
        phases = [[[pk[0],pk[1]*arguments.intensity,pk[2]] for pk in ph] for ph in phases]
    phases=selectPeakSubset(phases,arguments.p)
    
    
    #start positions of peaks
    q0=lambda x: 20.
    q1=lambda x: 24.
    q2=lambda x: 28.
    q3=lambda x: 30.

    #list of start position of the first peak in each phase    
    qbasisoffsets = [q0(None),q1(None),q2(None),q3(None)]
    
    #Functions that define the changes in each parameter, in each phase region, as a function of composition    
    r0_fdelta=lambda x: x[1]/.5
    r0_falpha=lambda x: x[0]-.25*r0_fdelta(x)
    r0_flambda=lambda x: 1.-r0_fdelta(x)-r0_falpha(x)
    
    r1_fdelta=lambda x: x[2]/.25
    r1_falpha=lambda x: x[0]-.25*r1_fdelta(x)
    r1_fbeta=lambda x: 1.0-r1_falpha(x)-r1_fdelta(x)
    
    r2_fdelta=lambda x: x[0]/.25
    r2_flambda=lambda x: x[2]-.25*r2_fdelta(x)
    r2_fbeta=lambda x: 1.0-r2_flambda(x)-r2_fdelta(x)
    
    r0_hdelta=lambda x: r0_fdelta(x)
    r0_halpha=lambda x: r0_falpha(x)
    r0_hlambda=lambda x: r0_flambda(x)
    
    r1_hdelta=lambda x: r1_fdelta(x)
    r1_halpha=lambda x: r1_falpha(x)
    r1_hbeta=lambda x: r1_fbeta(x)
    
    r2_hdelta=lambda x: r2_fdelta(x)
    r2_hlambda=lambda x: r2_flambda(x)
    r2_hbeta=lambda x: r2_fbeta(x)
    
    h0=lambda x: 0.
    w0=lambda x: 1.

    #Set of model functions to use for each phase in each phase region    
    regions = [                                 \
    [[q0, r0_halpha, w0], [q1, h0, w0], [q2, r0_hlambda, w0], [q3, r0_hdelta, w0]]                      ,\
    [[q0, r1_halpha, w0], [q1, r1_hbeta, w0], [q2, h0, w0], [q3, r1_hdelta, w0]]                      ,\
    [[q0, h0, w0], [q1, r2_hbeta, w0], [q2, r2_hlambda, w0], [q3, r2_hdelta, w0]]                      ,\
    ]
    
    #Object containing the results of the above manipulations
    return synthModel(compsize,regionchoice,qbasisoffsets,phases,regions,elements)
    
